/* anDOF
* Copyright (C) 2013, 2014 Harald Ulver
*
* This program is Free Software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as
* published by the Free Software Foundation, either version 3 of the
* License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program. If not, see <http://www.gnu.org/licenses/>.
*/

package org.quovadit.apps.andof;

import android.annotation.SuppressLint;
import android.app.Activity;
import android.net.Uri;
import android.os.Build;
import android.os.Bundle;
import android.widget.Button;
import android.widget.SeekBar;
import android.widget.TextView;
import android.widget.Toast;
import android.view.Menu;
import android.view.MenuItem;
import android.content.Intent;

import java.text.DecimalFormat;


public class anDOF extends Activity  implements SeekBar.OnSeekBarChangeListener{
	SeekBar mFocalSeekBar;
    SeekBar mApertureSeekBar;
    SeekBar mDistanceSeekBar;
    
    TextView mFocalText;
    TextView mApertureText;
    TextView mDistanceText;
    TextView mHyperfocal;
    TextView mMin;
    TextView mMax;
    TextView mDof;
    TextView mDist;
    Button mOptions;
    TextView mExit;
    

    // Data-Instance for get/set values
    public static Data myData;

    

    public void onStart() {
    	super.onStart();
    	
        // show current camera+lens on startup        
        Toast.makeText(getApplicationContext(), myData.getCameraName(-1)+"\n"+
        										myData.getLensName(-1),
 			   Toast.LENGTH_LONG).show();
    	
    }
    
    /** Called when the activity is first created. */
    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);

        myData = new Data(this);
		
		// read prefs on startup, otherwise coc is 0
		myData.readPrefs();
                
        if(Build.VERSION.SDK_INT > 10) hideIcon();

        setContentView(R.layout.main);

        
        // Text entries
        mHyperfocal = (TextView)findViewById(R.id.hyperfocal);
        mMin = (TextView)findViewById(R.id.min);
        mMax = (TextView)findViewById(R.id.max);
        mDof = (TextView)findViewById(R.id.dof);
        mDist = (TextView)findViewById(R.id.dist);
        
        // Distance widget
        mDistanceSeekBar = (SeekBar)findViewById(R.id.distance_seek);
        mDistanceSeekBar.setMax(myData.getDistanceCount());
        mDistanceSeekBar.setProgress(myData.getDistanceIndex());
        mDistanceSeekBar.setOnSeekBarChangeListener(this);
        mDistanceText = (TextView)findViewById(R.id.distance_text);
		mDistanceText.setText(getResources().getString(R.string.distance) +": " + myData.getDistanceName(-1));
        
        // Focal widgets
        mFocalSeekBar = (SeekBar)findViewById(R.id.focal_seek);
        mFocalSeekBar.setMax(myData.getFocalCount());
        mFocalSeekBar.setProgress(myData.getFocalIndex());
        mFocalSeekBar.setOnSeekBarChangeListener(this);
        mFocalText = (TextView)findViewById(R.id.focal_text);
		mFocalText.setText(getResources().getString(R.string.focal) + ": " + myData.getFocalName(-1) + " mm");
		if(myData.getFocalCount()==1) mFocalSeekBar.setVisibility(4); // no choice -> seekbar hidden

		// Aperture widgets
        mApertureSeekBar = (SeekBar)findViewById(R.id.aperture_seek);
        mApertureSeekBar.setMax(myData.getApertureCount());
        mApertureSeekBar.setProgress(myData.getApertureIndex());
        mApertureSeekBar.setOnSeekBarChangeListener(this);
        mApertureText = (TextView)findViewById(R.id.aperture_text);
		mApertureText.setText(getResources().getString(R.string.aperture) + ": " + myData.getApertureName(-1) + " ");
        
        calcDof();
        
        
        
    }
    
    public void onProgressChanged(SeekBar seekBar, int progress, boolean fromTouch) {

    	// prevent overshooting max-value by quickly sliding right		
		if(progress>=seekBar.getMax()) {
			progress=seekBar.getMax()-1;
			seekBar.setProgress(progress);
		}
		
		if (seekBar == mDistanceSeekBar) {
			myData.setDistanceIndex(progress);
			mDistanceText.setText(getResources().getString(R.string.distance) +": " + myData.getDistanceName(progress));
		}

		if (seekBar == mFocalSeekBar) {
			myData.setFocalIndex(progress);
			mFocalText.setText(getResources().getString(R.string.focal) + ": " + myData.getFocalName(progress) + " mm");
		}
			
    	if (seekBar == mApertureSeekBar) {
    		myData.setApertureIndex(progress);
    		mApertureText.setText(getResources().getString(R.string.aperture) + ": " + myData.getApertureName(progress) + " ");
    	}

    	calcDof();

    }

    /**
     * (Re-)calculates Dof and refreshes values shown
     * Called on startup and after input-changes
     * 
     */
    public void calcDof() {
		 Double CoC = myData.getCoc(-1);

		 Double focal = myData.getFocalNr(-1);

		 Double aperture = myData.getApertureNr(-1);
		 Double hyperFocal = (focal * focal) / (aperture * CoC) + focal;
		 
		 Double dofNear = 0.0;
		 Double dofFar  = 0.0;
		 Double distance;
		 int units = 1;

		 distance = myData.getDistanceNr(-1) * 10;
		 dofNear = ((hyperFocal - focal) * distance) / (hyperFocal + distance - (2*focal));

		// Prevent 'divide by zero' when calculating far distance.
		 if ( (hyperFocal - distance) <= 0.00001)
			 dofFar = 10000000.0; // set infinity at 10000m
		 else
			 dofFar = ((hyperFocal-focal) * distance) / (hyperFocal - distance);

		 // Convert depth of field numbers to proper units
		 dofNear = dofNear / 1000.0 / units;
		 dofFar  = dofFar / 1000.0 / units;

		 // convert hyperfocal distance to proper units
		 hyperFocal = hyperFocal / 1000.0 / units;
			 

		//Display results
		 DecimalFormat df = new DecimalFormat("0.00");  // show decimal even for even numbers
		 DecimalFormat dfi = new DecimalFormat("#");    // no decimal for large numbers

		 mHyperfocal.setText(getResources().getString(R.string.hyperfocal) + ": " + df.format(hyperFocal) + " m ");
		 
		 mDist.setText(myData.getDistanceName(-1));
		 
		 mMin.setText(df.format(dofNear) + " m");
		 
		 if (dofFar >= 10000.0 ) {// set infinity at 10000m
			 mMax.setText("Infinity ");
			 mDof.setText("Infinity ");
		 } else if (dofFar >= 100) {
			 mMax.setText(dfi.format(dofFar) + " m");
			 mDof.setText(dfi.format(dofFar - dofNear) + " m");
		 } else {
			 mMax.setText(df.format(dofFar) + " m");
			 mDof.setText(df.format(dofFar - dofNear) + " m");
		 }
		 		 
    }
    
    protected void onPause() {
        super.onPause();

        myData.savePrefs();

    }

     public void onStartTrackingTouch(SeekBar seekBar) {
     }

     public void onStopTrackingTouch(SeekBar seekBar) {
     }
     
     @Override public boolean onCreateOptionsMenu(Menu menu) {
         getMenuInflater().inflate(R.menu.main, menu);
         return true;
     }

     @Override
     public boolean onPrepareOptionsMenu(Menu menu) {

    	 refreshCloseup(menu.findItem(R.id.closeup));
    	 
    	 return super.onPrepareOptionsMenu(menu);

     }
     
     @Override public boolean onOptionsItemSelected(MenuItem item) {

    	 Intent intent;
			switch (item.getItemId()) {
            case R.id.currentlens:
            	intent = new Intent(this, CurrentLens.class);
            	startActivity(intent);  
				finish();
				return true;
			
            case R.id.closeup:
            	myData.changeCloseupEnabled();

            	refreshCloseup(item);
            	myData.setDistanceIndex(0);
            	myData.savePrefs();
            	this.onCreate(null);
            	
            	Toast.makeText(getApplicationContext(), getCloseupToast(),
          			   Toast.LENGTH_LONG).show();

                return true;
			
			case R.id.mycamera:
            	intent = new Intent(this, MyCamera.class);
            	startActivity(intent);  
				finish();
				return true;

            case R.id.mylenses:
            	intent = new Intent(this, MyLenses.class);
            	startActivity(intent);  
				finish();
				return true;

            case R.id.resetsettings:
            	myData.resetSettings();
            	finish();
            	startActivity(getIntent());
            	
            	return true;

            case R.id.help:
            	Intent browserIntent = new Intent(Intent.ACTION_VIEW, Uri.parse("http://andof.quovadit.org"));
            	startActivity(browserIntent);
            	return true;

            default:
                return super.onOptionsItemSelected(item);
        }

     }
     
     private void refreshCloseup(MenuItem item) {
    	 
    	 if(myData.getCloseupEnabled()){
    		 item.setIcon(R.drawable.closeup);
    	 }else{
    		 item.setIcon(R.drawable.closeup_off);
    	 }
    	 
     }

     private String getCloseupToast() {

    	 String closeupToast;
    	 
    	 if(myData.getCloseupEnabled()){
    		 closeupToast = getResources().getString(R.string.closeup_on);
    	 }else{
    		 closeupToast = getResources().getString(R.string.closeup_off);
    	 }
    	 
    	 return closeupToast;
     }

     @SuppressLint("NewApi")
	private void hideIcon() {
         getActionBar().setDisplayShowHomeEnabled(false); 
     }
     
     
}



